# GOV.UK Mini Environment Admin
# Copyright © 2018  Christopher Baines <mail@cbaines.net>
#
# This file is part of the GOV.UK Mini Environment Admin.
#
# The GOV.UK Mini Environment Admin is free software: you can
# redistribute it and/or modify it under the terms of the GNU Affero
# General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later
# version.
#
# The GOV.UK Mini Environment Admin is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with the GOV.UK Mini Environment Admin.  If not, see
# <http://www.gnu.org/licenses/>.

# == Schema Information
#
# Table name: mini_environments
#
#  id                     :integer          not null, primary key
#  name                   :string
#  created_at             :datetime         not null
#  updated_at             :datetime         not null
#  info                   :jsonb
#  govuk_guix_revision_id :string
#  backend_type           :string
#  backend_id             :integer
#  backend_data           :jsonb
#  signon_users           :jsonb
#  data_snapshot_id       :integer
#

class MiniEnvironment < ApplicationRecord
  include Status

  belongs_to :govuk_guix_revision, class_name: 'GovukGuix::Revision'
  belongs_to :backend, polymorphic: true

  has_many :services, class_name: 'MiniEnvironmentService', dependent: :destroy
  belongs_to(
    :data_snapshot,
    class_name: 'GovukGuix::DataSnapshot',
    optional: true
  )

  def enqueued_terraform_jobs
    Que.execute("SELECT * FROM que_jobs WHERE args->>0 = '#{id}'")
  end

  def signon_user_openstructs
    signon_users.map do |signon_user|
      OpenStruct.new(
        signon_user.merge(
          'persisted?' => false
        )
      )
    end
  end

  def terraform_states
    backend.terraform_states(self)
  end

  def signon_users_attributes=(attributes)
    self.signon_users = attributes.sort.map(&:last)
  end
end
